#!/bin/bash

#set -e
###################################################
###
### README：运行此脚本之前，先确保做以下操作已完成：
### 1、创建sysadm、secadm、audadm三大管理员用户
### 2、创建/etc/deepin-security-enhance、/etc/deepin-security-enhance/config和/etc/deepin-security-enhance/script目录
### 3、将配置文件都拷贝到/etc/deepin-security-enhance/config目录，将secadm_chown.sh（权限改为0755）拷贝到/etc/deepin-security-enhance/script目录
### 4、将login_counts和alertd文件拷贝到/usr/bin目录
### 5、将security-enhance.service文件拷贝到/usr/lib/systemd/system目录
### 6、本脚本自己（权限改为0755）也拷贝到拷贝到/etc/deepin-security-enhance/script目录下

###################################################
### dbus开关命令：
### 开启：dbus-send --system --print-reply --dest=com.deepin.daemon.SecurityEnhance  /com/deepin/daemon/SecurityEnhance com.deepin.daemon.SecurityEnhance.Enable boolean:true boolean:true/false string:sysadmpasswd string:secadmpasswd string:audadmpasswd
### 关闭：dbus-send --system --print-reply --dest=com.deepin.daemon.SecurityEnhance  /com/deepin/daemon/SecurityEnhance com.deepin.daemon.SecurityEnhance.Enable boolean:false boolean:true/false string:  string:  string: 
### 查询：dbus-send --system --print-reply --dest=com.deepin.daemon.SecurityEnhance  /com/deepin/daemon/SecurityEnhance com.deepin.daemon.SecurityEnhance.Status

###################################################
### 执行此脚本参数定义：
### $1:enable/disable 开启/关闭等保三级
### $2:true/false 是否删除已有管理员账号，关闭等保三级时有效，开启等保三级时忽略此参数
### $3:系统管理员密码
### $4:安全管理员密码
### $5:审计管理员密码
### $6:是否开启等保四级,false:不开启,true:开启    (预留功能，暂不支持)

SCRIPT_NAME=$(basename "$0")
ROOT_USER_ID=`id | grep "uid=0(root)"`
DEEPIN_SECURITY_DIR="/etc/deepin-security-enhance"
DS_BACKUP_DIR=${DEEPIN_SECURITY_DIR}"/back_up"
DS_CONFIG_DIR=${DEEPIN_SECURITY_DIR}"/config"
DS_SCRIPT_DIR=${DEEPIN_SECURITY_DIR}"/script"
SELINUX_DEFAULT_MLS_FILE="/etc/selinux/mls/policy/policy.*"
GRUB_CFG_DIR="/etc/default/grub.d"
GRUB_CMDLINE_SELINUX_FILE="16_deepin_selinux.cfg"
UNLABELED_FLAG_FILE=".unlabeled.tmp"
SYSTEMD_SERVICE_DIR="/lib/systemd/system"
SECURITY_ENHANCE_SERVICE="security-enhance.service"
AUDITD_SERVICE="auditd.service"
SECURITY_ENHANCE_SERVICE_PATH=$SYSTEMD_SERVICE_DIR"/"$SECURITY_ENHANCE_SERVICE
AUDITD_SERVICE_PATH=$SYSTEMD_SERVICE_DIR"/"$AUDITD_SERVICE
PERMISSION_MANAGER_SERVICE_PATH="/lib/systemd/system/permission-manager.service"
DEFAULT_SEUSERS_LOCAL="/var/lib/selinux/default/active/seusers.local"
MLS_SEUSERS_LOCAL="/var/lib/selinux/mls/active/seusers.local"
SYS_ADM="sysadm"
SEC_ADM="secadm"
AUD_ADM="audadm"

# 三大管理员的加密密码，只会在enable的时候用到
SYS_PASSWD=$3
SEC_PASSWD=$4
AUD_PASSWD=$5

# 脚本异常卡死时，再运行脚本时，会检查是否有正在运行的脚本，如果有，则杀掉
SCRIPT_PID=$$
DS_STATE_DIR=${DEEPIN_SECURITY_DIR}"/state"
DS_STATE_PID_FILE=${DS_STATE_DIR}"/Pid"
DS_STATE_ENABLE_FILE=${DS_STATE_DIR}"/Enable"

DS_STATE_STATUS_FILE=${DS_STATE_DIR}"/Status"
STATUS_OPEN="open"
STATUS_OPENING="opening"
STATUS_CLOSE="close"
STATUS_CLOSING="closing"
STATUS_UPDATE_GRUB="grub_updating"

# 是否开启等保四级 (预留功能，暂不支持)
#SECURITY_LEVEL_FOUR=$6
SECURITY_LEVEL_FOUR="false"

SELINUX_STATUS=`getenforce`

DEEPIN_PW_CHECK=`dpkg -l | grep "ii  deepin-pw-check"`
DEEPIN_AUTHENTICATE=`dpkg -l | grep "ii  deepin-authenticate"`
PAM_PWQUALITY_PKG_CHECK_SUCCESS=`dpkg -l | grep "ii  libpam-pwquality"`
PAM_CRACKLIB_PKG_CHECK_SUCCESS=`dpkg -l | grep "ii  libpam-cracklib"`

# 是否使用了特权运行此脚本
if [ -z "$ROOT_USER_ID" ];then
	echo -n '-1'
	exit 0
fi

# 检查安全目录
if [ ! -d ${DEEPIN_SECURITY_DIR} ];then
	echo -n '-2'
	exit 0
fi

if [ ! -d ${DS_CONFIG_DIR} ];then
	echo -n '-3'
	exit 0
fi

if [ ! -d ${DS_SCRIPT_DIR} ];then
	echo -n '-4'
	exit 0
fi

function back_up_config()
{
	if [ ! -d ${DS_BACKUP_DIR} ];then
		mkdir ${DS_BACKUP_DIR}
	fi

	if [ -z "$DEEPIN_PW_CHECK" ];then
		if [ -f /etc/security/pwquality.conf ];then
			cp /etc/security/pwquality.conf ${DS_BACKUP_DIR}/pam_pwquality.conf > /dev/null 2>&1
		fi

		if [ -f /etc/pam.d/common-password ];then
			cp /etc/pam.d/common-password ${DS_BACKUP_DIR}/pam_common-password > /dev/null 2>&1
		fi
	fi

	if [ -z "$DEEPIN_AUTHENTICATE" ];then
		if [ -f /etc/pam.d/sshd ];then
			cp /etc/pam.d/sshd ${DS_BACKUP_DIR}/pam_sshd > /dev/null 2>&1
		fi

		if [ -f /etc/pam.d/login ];then
			cp /etc/pam.d/login ${DS_BACKUP_DIR}/pam_login > /dev/null 2>&1
		fi

		if [ -f /etc/pam.d/lightdm ];then
			cp /etc/pam.d/lightdm ${DS_BACKUP_DIR}/pam_lightdm > /dev/null 2>&1
		fi
	fi

	if [ -f /etc/selinux/config ];then
		cp /etc/selinux/config ${DS_BACKUP_DIR}/selinux_config > /dev/null 2>&1
	fi

	if [ $SECURITY_LEVEL_FOUR = "true" ];then
		#wkhtmltopdf(等保四级要求)
		if [ -f /usr/bin/wkhtmltopdf ];then
			cp /usr/bin/wkhtmltopdf ${DS_BACKUP_DIR}/wkhtmltopdf > /dev/null 2>&1
		fi

		#sysrq(等保四级要求)
		if [ -f /etc/sysctl.conf ];then
			cp /etc/sysctl.conf ${DS_BACKUP_DIR}/sysctl.conf > /dev/null 2>&1
		fi
	fi
}

function restore_config()
{
	# 超时自动退出
	if [ -f /etc/profile ];then
		sed -i '/^TMOUT=120/d' /etc/profile > /dev/null 2>&1
	fi

	# 登录次数信息打印功能
	if [ -f /var/log/auth.log ];then
		# 还原auth.log文件权限
		chmod 0640 /var/log/auth.log > /dev/null 2>&1
	fi
	if [ -f /etc/bash.bashrc ];then
		sed -i '/^login_counts/d' /etc/bash.bashrc > /dev/null 2>&1
	fi

	if [ -z "$DEEPIN_PW_CHECK" ];then
		if [ -f ${DS_BACKUP_DIR}/pam_pwquality.conf ];then
			cp ${DS_BACKUP_DIR}/pam_pwquality.conf /etc/security/pwquality.conf > /dev/null 2>&1
		fi

		if [ -f ${DS_BACKUP_DIR}/pam_common-password ];then
			cp ${DS_BACKUP_DIR}/pam_common-password /etc/pam.d/common-password > /dev/null 2>&1
		fi
	fi

	if [ -z "$DEEPIN_AUTHENTICATE" ];then
		if [ -f ${DS_BACKUP_DIR}/pam_sshd ];then
			cp ${DS_BACKUP_DIR}/pam_sshd /etc/pam.d/sshd > /dev/null 2>&1
		fi

		if [ -f ${DS_BACKUP_DIR}/pam_login ];then
			cp ${DS_BACKUP_DIR}/pam_login /etc/pam.d/login > /dev/null 2>&1
		fi

		if [ -f ${DS_BACKUP_DIR}/pam_lightdm ];then
			cp ${DS_BACKUP_DIR}/pam_lightdm /etc/pam.d/lightdm > /dev/null 2>&1
		fi
	fi

	if [ -f ${DS_BACKUP_DIR}/system_logind.conf ];then
		cp ${DS_BACKUP_DIR}/system_logind.conf /etc/systemd/logind.conf > /dev/null 2>&1
	fi

	if [ -f ${DS_BACKUP_DIR}/selinux_config ];then
		cp ${DS_BACKUP_DIR}/selinux_config /etc/selinux/config > /dev/null 2>&1
	fi

	if [ $SECURITY_LEVEL_FOUR = "true" ];then
		#wkhtmltopdf(等保四级要求)
		if [ -f ${DS_BACKUP_DIR}/wkhtmltopdf ];then
			cp ${DS_BACKUP_DIR}/wkhtmltopdf /usr/bin/wkhtmltopdf > /dev/null 2>&1
		fi

		#sysrq(等保四级要求)
		if [ -f ${DS_BACKUP_DIR}/sysctl.conf ];then
			cp ${DS_BACKUP_DIR}/sysctl.conf /etc/sysctl.conf > /dev/null 2>&1
		fi
	fi

	if [ -d ${DS_BACKUP_DIR} ];then
		rm -rf ${DS_BACKUP_DIR} > /dev/null 2>&1
	fi
}

function restore_root()
{
	# 去除SELinux自动打标签以及还未打标签标志
	if [ -f /.autorelabel ];then
		rm /.autorelabel > /dev/null 2>&1
	fi
	if [ -f ${DEEPIN_SECURITY_DIR}/${UNLABELED_FLAG_FILE} ];then
		rm ${DEEPIN_SECURITY_DIR}/${UNLABELED_FLAG_FILE} > /dev/null 2>&1
	fi

	if [ $SECURITY_LEVEL_FOUR = "true" ];then
		# 确保是在mls模式下
		sed -i 's/^SELINUXTYPE=.*/SELINUXTYPE=mls/g' /etc/selinux/config

		TMP_MLS_SEUSERS_LOCAL="/tmp/mls_seusers.local"
		cp $MLS_SEUSERS_LOCAL $TMP_MLS_SEUSERS_LOCAL > /dev/null 2>&1
		sed -i "/${SYS_ADM}:/d" $TMP_MLS_SEUSERS_LOCAL > /dev/null 2>&1
		sed -i "/${SEC_ADM}:/d" $TMP_MLS_SEUSERS_LOCAL > /dev/null 2>&1
		sed -i "/${AUD_ADM}:/d" $TMP_MLS_SEUSERS_LOCAL > /dev/null 2>&1
		sed -i "/root:system_u/d" $TMP_MLS_SEUSERS_LOCAL > /dev/null 2>&1
		cp $TMP_MLS_SEUSERS_LOCAL $MLS_SEUSERS_LOCAL > /dev/null 2>&1
		semanage login -m -s root __default__ > /dev/null 2>&1
		rm $TMP_MLS_SEUSERS_LOCAL > /dev/null 2>&1
	fi

	# 确保是在default模式下
	sed -i 's/^SELINUXTYPE=.*/SELINUXTYPE=default/g' /etc/selinux/config

	TMP_SEUSERS_LOCAL="/tmp/seusers.local"
	cp $DEFAULT_SEUSERS_LOCAL $TMP_SEUSERS_LOCAL > /dev/null 2>&1
	sed -i "/${SYS_ADM}:/d" $TMP_SEUSERS_LOCAL > /dev/null 2>&1

	sed -i "/${SEC_ADM}:/d" $TMP_SEUSERS_LOCAL > /dev/null 2>&1
	sed -i "/${AUD_ADM}:/d" $TMP_SEUSERS_LOCAL > /dev/null 2>&1
	sed -i "/root:system_u/d" $TMP_SEUSERS_LOCAL > /dev/null 2>&1
	cp $TMP_SEUSERS_LOCAL $DEFAULT_SEUSERS_LOCAL > /dev/null 2>&1
	semanage login -m -s root __default__ > /dev/null 2>&1
	rm $TMP_SEUSERS_LOCAL > /dev/null 2>&1

	usermod -U root > /dev/null 2>&1
	chown -R 0:0 /var/lib/selinux > /dev/null 2>&1
	chown -R 0:0 /var/lib/sepolgen > /dev/null 2>&1
	chown -R 0:0 /etc/selinux > /dev/null 2>&1
	chown -R 0:0 /usr/share/selinux > /dev/null 2>&1
	chmod -s /sbin/setfiles > /dev/null 2>&1
	chmod -s /sbin/restorecon > /dev/null 2>&1
	chmod -s /sbin/semodule > /dev/null 2>&1

	chown -R 0:0 /etc/audit > /dev/null 2>&1
	chown -R 0:0 /etc/audisp > /dev/null 2>&1

	chcon -R -u root /home/* > /dev/null 2>&1

	if [ -f $SECURITY_ENHANCE_SERVICE_PATH ];then
		systemctl disable $SECURITY_ENHANCE_SERVICE > /dev/null 2>&1
	fi
}

function delete_account()
{
	# 检查管理员是否存在
	if  id -u $SEC_ADM > /dev/null 2>&1;then
		usermod -s /usr/sbin/nologin $SEC_ADM > /dev/null 2>&1
	fi
	if  id -u $AUD_ADM > /dev/null 2>&1;then
		usermod -G $AUD_ADM -s /usr/sbin/nologin -L $AUD_ADM > /dev/null 2>&1
	fi

	if [ $1 = "true" ];then
		# 删除管理员
		if id -u $SEC_ADM > /dev/null 2>&1;then
			userdel -r $SEC_ADM > /dev/null 2>&1
		fi
		if id -u $AUD_ADM > /dev/null 2>&1;then
			userdel -r $AUD_ADM > /dev/null 2>&1
		fi
	fi
}

function restore_all()
{
	if [ -f ${GRUB_CFG_DIR}/${GRUB_CMDLINE_SELINUX_FILE} ];then
		rm ${GRUB_CFG_DIR}/${GRUB_CMDLINE_SELINUX_FILE} > /dev/null 2>&1
	fi

	restore_root
	delete_account "true"

	# 配置还原必须放最后，保证配置还原后不会再误修改
	restore_config
}

# 开启等保出错时，用于还原配置并退出
function exit_and_restore()
{
	restore_all
	echo "$STATUS_CLOSE" | tee ${DS_STATE_STATUS_FILE} > /dev/null 2>&1
	rm ${DS_STATE_PID_FILE} > /dev/null 2>&1
	exit 0
}

function identity_authentication()
{
	## 强化口令
	#先查看是否有pam_deepin_pw_check模块，如果有，则由上层调用pam模块接口配置，如果没有，则使用pam_pwquality.so，在此脚本里手动配置
	if [ -z "$DEEPIN_PW_CHECK" ];then
		if [ -z "$PAM_PWQUALITY_PKG_CHECK_SUCCESS" ];then
			if [ -z "$PAM_CRACKLIB_PKG_CHECK_SUCCESS" ];then
				echo -n '-101'
				exit_and_restore
			fi

			if [ -f ${DS_CONFIG_DIR}/pam_common-password_cracklib ];then
				cp ${DS_CONFIG_DIR}/pam_common-password_cracklib /etc/pam.d/common-password > /dev/null 2>&1
				if [ $? != 0 ];then
					echo -n '-102'
					exit_and_restore
				fi
			else
				#sed -i '/pam_cracklib.so/d' /etc/pam.d/common-password
				#sed -i '25ipassword required pam_cracklib.so retry=3 minlen=8' /etc/pam.d/common-password
				echo -n '-103'
				exit_and_restore
			fi
		else
			if [ -f ${DS_CONFIG_DIR}/pam_common-password_pwquality ];then
				cp ${DS_CONFIG_DIR}/pam_common-password_pwquality /etc/pam.d/common-password > /dev/null 2>&1
				if [ $? != 0 ];then
					echo -n '-104'
					exit_and_restore
				fi
			else
				#sed -i '/pam_pwquality.so/d' /etc/pam.d/common-password
				#sed -i '25ipassword     requisite       pam_pwquality.so retry=3' /etc/pam.d/common-password
				echo -n '-105'
				exit_and_restore
			fi

			if [ -f ${DS_CONFIG_DIR}/pam_pwquality.conf ];then
				cp ${DS_CONFIG_DIR}/pam_pwquality.conf /etc/security/pwquality.conf > /dev/null 2>&1
				if [ $? != 0 ];then
					echo -n '-106'
					exit_and_restore
				fi
			else
				echo -n '-107'
				exit_and_restore
			fi
		fi
	fi

	## 认证策略
	#先查看是否有pam_deepin_authentication.so模块，如果有，则由上层调用pam_deepin_authentication接口自行配置，如果没有，则使用pam_tally2.so，在此脚本里手动配置
	if [ -z "$DEEPIN_AUTHENTICATE" ];then
		if [ -f ${DS_CONFIG_DIR}/pam_sshd ];then
			cp ${DS_CONFIG_DIR}/pam_sshd /etc/pam.d/sshd > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-108'
				exit_and_restore
			fi
		else
			#sed -i '/pam_tally2.so/d' /etc/pam.d/sshd
			#sed -i '2iauth     required       pam_tally2.so deny=3 unlock_time=1200' /etc/pam.d/sshd
			echo -n '-109'
			exit_and_restore
		fi

		if [ -f ${DS_CONFIG_DIR}/pam_login ];then
			cp ${DS_CONFIG_DIR}/pam_login /etc/pam.d/login > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-110'
				exit_and_restore
			fi
		else
			#sed -i '/pam_tally2.so/d' /etc/pam.d/login
			#sed -i '2iauth     required       pam_tally2.so deny=3 unlock_time=1200' /etc/pam.d/login
			echo -n '-111'
			exit_and_restore
		fi

		if [ -f ${DS_CONFIG_DIR}/pam_lightdm ];then
			cp ${DS_CONFIG_DIR}/pam_lightdm /etc/pam.d/lightdm > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-112'
				exit_and_restore
			fi
		else
			#sed -i '/pam_tally2.so/d' /etc/pam.d/lightdm
			#sed -i '2iauth     required       pam_tally2.so deny=3 unlock_time=1200' /etc/pam.d/lightdm
			echo -n '-113'
			exit_and_restore
		fi
	fi
}

function label_and_MAC()
{
	case "$1" in
	1)
		############# 第一阶段，创建管理员账号，并设置密码 #############
		# 创建系统管理员并设置密码,传下来的密码是经过加密的，故要加-p参数
		if ! id -u $SYS_ADM > /dev/null 2>&1;then
			useradd -m $SYS_ADM -s /bin/bash -p $SYS_PASSWD > /dev/null 2>&1
		else
			usermod $SYS_ADM -s /bin/bash -p $SYS_PASSWD > /dev/null 2>&1
		fi
		if [ $? != 0 ];then
			echo -n '-301'
			exit_and_restore
		fi

		# 创建安全管理员并设置密码,传下来的密码是经过加密的，故要加-p参数
		if ! id -u $SEC_ADM > /dev/null 2>&1;then
			useradd -m $SEC_ADM -s /bin/bash -p $SEC_PASSWD > /dev/null 2>&1
		else
			usermod $SEC_ADM -s /bin/bash -p $SEC_PASSWD > /dev/null 2>&1
		fi
		if [ $? != 0 ];then
			echo -n '-302'
			exit_and_restore
		fi

		# 创建审计管理员并设置密码,传下来的密码是经过加密的，故要加-p参数
		if ! id -u $AUD_ADM > /dev/null 2>&1;then
			useradd -m $AUD_ADM -s /bin/bash -p $AUD_PASSWD > /dev/null 2>&1
		else
			usermod $AUD_ADM -s /bin/bash -p $AUD_PASSWD > /dev/null 2>&1
		fi
		if [ $? != 0 ];then
			echo -n '-303'
			exit_and_restore
		fi

		# 锁定原root用户
		usermod -L root > /dev/null 2>&1
	;;
	2)
		############# 第二阶段，设置管理员账号MAC权限，初步开启三权分立 #############
		SELINUX_DEFAULT_POLICY_FILE=`ls /etc/selinux/default/policy/policy.* 2>/dev/null`
		# 检查是否安装了selinux policy包
		if [ -z "$SELINUX_DEFAULT_POLICY_FILE" ];then
			echo -n '-304'
			exit_and_restore
		fi
		# 默认启用selinux
		if [ -f ${DS_CONFIG_DIR}/selinux_config ];then
			cp ${DS_CONFIG_DIR}/selinux_config /etc/selinux/config > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-305'
				exit_and_restore
			fi
		else
			#sed -i 's/^SELINUX=permissive/SELINUX=enforcing/g' /etc/selinux/config
			echo -n '-306'
			exit_and_restore
		fi

		# selinux三权分立配置
		TMP_SEUSERS_LOCAL="/tmp/seusers.local"
		cp $DEFAULT_SEUSERS_LOCAL $TMP_SEUSERS_LOCAL > /dev/null 2>&1
		str_user="$SYS_ADM $SEC_ADM $AUD_ADM"
		for i in ${str_user}
		do
			TMP="${i}:${i}_u:s0"
			echo $TMP >> $TMP_SEUSERS_LOCAL
			if [ $? != 0 ];then
				echo -n '-307'
				exit_and_restore
			fi
		done
		echo root:system_u:s0 >> $TMP_SEUSERS_LOCAL
		if [ $? != 0 ];then
			echo -n '-308'
			exit_and_restore
		fi
		cp $TMP_SEUSERS_LOCAL $DEFAULT_SEUSERS_LOCAL > /dev/null 2>&1
		semanage login -m -s user_u __default__ > /dev/null 2>&1
		rm $TMP_SEUSERS_LOCAL > /dev/null 2>&1

		#若开启的是等保四级，则进入mls模式
		if [ $SECURITY_LEVEL_FOUR = "true" ];then
			#若开启的是等保四级，则进入mls模式
			sed -i 's/SELINUX=enforcing/SELINUX=permissive/g' /etc/selinux/config
			sed -i 's/SELINUXTYPE=default/SELINUXTYPE=mls/g' /etc/selinux/config

			###selinux三权分立配置
			TMP_MLS_SEUSERS_LOCAL="/tmp/mls_seusers.local"
			cp $MLS_SEUSERS_LOCAL $TMP_MLS_SEUSERS_LOCAL > /dev/null 2>&1
			mls_user="$SYS_ADM $SEC_ADM"
			for i in ${mls_user}
			do
				TMP="${i}:${i}_u:s0"
				echo $TMP >> $TMP_MLS_SEUSERS_LOCAL
				if [ $? != 0 ];then
					echo -n '-309'
					exit_and_restore
				fi
			done
			echo ${AUD_ADM}:${AUD_ADM}_u:s15:c0.c1023-s15:c0.c1023 >> $TMP_MLS_SEUSERS_LOCAL
			if [ $? != 0 ];then
				echo -n '-310'
				exit_and_restore
			fi
			cp $TMP_MLS_SEUSERS_LOCAL $MLS_SEUSERS_LOCAL > /dev/null 2>&1
			rm $TMP_MLS_SEUSERS_LOCAL > /dev/null 2>&1

			semanage login -m -s user_u __default__ > /dev/null 2>&1
			semanage login -m -s system_u root > /dev/null 2>&1
		fi
	;;
	3)
		############# 第三阶段，设置管理员账号dac权限，完成三权分立 #############
		#保证管理员目录下文件的属主
		if [ -d /home/${SYS_ADM} ];then
			chown -R $SYS_ADM /home/${SYS_ADM} > /dev/null 2>&1
			chmod o+rx /home/${SYS_ADM} > /dev/null 2>&1
		fi
		if [ -d /home/${SEC_ADM} ];then
			chown -R $SEC_ADM /home/${SEC_ADM} > /dev/null 2>&1
			chmod o+rx /home/${SEC_ADM} > /dev/null 2>&1
		fi
		if [ -d /home/${AUD_ADM} ];then
			chown -R $AUD_ADM /home/${AUD_ADM} > /dev/null 2>&1
			chmod o+rx /home/${AUD_ADM} > /dev/null 2>&1
		fi

		# 系统管理员dac权限配置
		usermod $SYS_ADM -aG sudo,adm > /dev/null 2>&1
		if [ $? != 0 ];then
			echo -n '-311'
			exit_and_restore
		fi

		# 安全管理员dac权限配置
		if [ -d /var/lib/selinux ];then
			chown -R ${SEC_ADM}:${SEC_ADM} /var/lib/selinux > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-312'
				exit_and_restore
			fi
		fi
		if [ -d /var/lib/sepolgen ];then
			chown -R ${SEC_ADM}:${SEC_ADM} /var/lib/sepolgen > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-313'
				exit_and_restore
			fi
		fi
		if [ -d /etc/selinux ];then
			chown -R ${SEC_ADM}:${SEC_ADM} /etc/selinux > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-314'
				exit_and_restore
			fi
		fi
		if [ -d /usr/share/selinux ];then
			chown -R ${SEC_ADM}:${SEC_ADM} /usr/share/selinux > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-315'
				exit_and_restore
			fi
		fi

		if [ -f /sbin/setfiles ];then
			chmod +s /sbin/setfiles > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-316'
				exit_and_restore
			fi
		fi
		if [ -f /sbin/restorecon ];then
			chmod +s /sbin/restorecon > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-317'
				exit_and_restore
			fi
		fi
		if [ -f /sbin/semodule ];then
			chmod +s /sbin/semodule > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-318'
				exit_and_restore
			fi
		fi

		# 审计管理员dac权限配置
		usermod $AUD_ADM -aG sudo,adm > /dev/null 2>&1
		if [ $? != 0 ];then
			echo -n '-319'
			exit_and_restore
		fi
		if [ -d /etc/audit ];then
			chown -R 0:adm /etc/audit > /dev/null 2>&1
			chmod -R g+w /etc/audit > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-320'
				exit_and_restore
			fi
		fi
		if [ -d /etc/audisp ];then
			chown -R 0:adm  /etc/audisp > /dev/null 2>&1
			chmod -R g+w /etc/audisp > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-321'
				exit_and_restore
			fi
		fi

		# 确保安全管理员和审计管理员每次开机都拥有正确的DAC权限
		if [ -f $SECURITY_ENHANCE_SERVICE_PATH ];then
			systemctl enable $SECURITY_ENHANCE_SERVICE > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-322'
				exit_and_restore
			fi
		else
			echo -n '-323'
			exit_and_restore
		fi
	;;
	4)
		############# 第四阶段, 设置打印水印 (仅等保四级要求) #############
		if [ $SECURITY_LEVEL_FOUR = "true" ];then
			if [ ! -d /usr/lib/cups/filter/ ];then
				mkdir -p /usr/lib/cups/filter > /dev/null 2>&1
			fi

			if [ -f ${DS_CONFIG_DIR}/watermark.py ];then
				cp ${DS_CONFIG_DIR}/watermark.py /usr/lib/cups/filter/watermarkpdf > /dev/null 2>&1
				if [ $? != 0 ];then
					echo -n '-324'
					exit_and_restore
				fi
			else
				echo -n '-325'
				exit_and_restore
			fi
			chmod 755 /usr/lib/cups/filter/watermarkpdf > /dev/null 2>&1

			#cp -R /usr/share/deepin-security/watermark/home/* /home/

			if [ -f /usr/bin/wkhtmltopdf ];then
				#  新的wkhtmltopdf会调用老的，因此需要备份老的
				mv /usr/bin/wkhtmltopdf /usr/bin/wkhtmltopdf.old
			fi

			if [ -f ${DS_CONFIG_DIR}/wkhtmltopdf ];then
				cp ${DS_CONFIG_DIR}/wkhtmltopdf  /usr/bin/wkhtmltopdf > /dev/null 2>&1
				if [ $? != 0 ];then
					echo -n '-326'
					exit_and_restore
				fi
			else
				echo -n '-327'
				exit_and_restore
			fi
			chmod +x /usr/bin/wkhtmltopdf > /dev/null 2>&1
		fi
	;;
	5)
		############# 第五阶段，开启SELinux #############
		# 修改grub
		echo "$STATUS_UPDATE_GRUB" | tee -a ${DS_STATE_STATUS_FILE} > /dev/null 2>&1
		if [ -d ${GRUB_CFG_DIR} ];then
			echo "GRUB_CMDLINE_LINUX_DEFAULT=\"\$GRUB_CMDLINE_LINUX_DEFAULT security=selinux checkreqprot=1\"" | tee ${GRUB_CFG_DIR}/${GRUB_CMDLINE_SELINUX_FILE} > /dev/null 2>&1
			if [ $? != 0 ];then
				echo -n '-329'
				exit_and_restore
			fi
			# 若已经启动了SELinux，则不需要再更新grub了, 这样可以优化启动速度
			if [ $SELINUX_STATUS = "Disabled" ];then
				update-grub > /dev/null 2>&1
				if [ $? != 0 ];then
					rm ${GRUB_CFG_DIR}/${GRUB_CMDLINE_SELINUX_FILE} > /dev/null 2>&1
					echo -n '-330'
					exit_and_restore
				fi
			fi
		fi
		# SELinux自动打标签
		fixfiles -F onboot > /dev/null 2>&1
		if [ $? != 0 ];then
			echo -n '-328'
			exit_and_restore
		fi
		# 由于现在无法设置bool，因此关闭黑名单模式放到secadm_chown.sh脚本中进行
	;;
	esac
}

function security_audit()
{
	# 检查是否安装了auditd
	AUDITD_PKG_CHECK_SUCCESS=`dpkg -l | grep "ii  auditd"`
	if [ -z "$AUDITD_PKG_CHECK_SUCCESS" ];then
		echo -n '-401'
		exit_and_restore
	fi

	# 开启审计服务
	if [ -f $AUDITD_SERVICE_PATH ];then
		systemctl enable $AUDITD_SERVICE > /dev/null 2>&1
		if [ $? != 0 ];then
			echo -n '-402'
			exit_and_restore
		fi
	else
		echo -n '-403'
		exit_and_restore
	fi
}

function login_access_control()
{
	# 系统默认就限制只开启6个tty，可通过配置/etc/systemd/logind.conf中的NAutoVTs字段的值来修改

	# 打印用户上次登录的信息以及登陆成功/失败次数统计(login、sshd文件已在已在身份鉴别阶段一起配置好)
	#sed -i 's/pam_lastlog.so/pam_lastlog.so showfailed/g' /etc/pam.d/login
	#sed -i 'N;42isession    optional     pam_lastlog.so showfailed' /etc/pam.d/sshd

	if [ ! -f /etc/.lasts ];then
		touch /etc/.lasts > /dev/null 2>&1
		if [ $? != 0 ];then
			echo -n '-1001'
			exit_and_restore
		fi
	fi
	if [ ! -f /etc/.lastsb ];then
		touch /etc/.lastsb > /dev/null 2>&1
		if [ $? != 0 ];then
			echo -n '-1002'
			exit_and_restore
		fi
	fi
	chmod 0666 /etc/.lasts > /dev/null 2>&1
	chmod 0666 /etc/.lastsb > /dev/null 2>&1
	if [ -f /var/log/auth.log ];then
		chmod 0644 /var/log/auth.log > /dev/null 2>&1
	fi

	if [ -f /etc/bash.bashrc ];then
		sed -i '/^login_counts/d' /etc/bash.bashrc
		echo "login_counts" >> /etc/bash.bashrc
	fi

	# 超时自动退出
	if [ -f /etc/profile ];then
		echo "TMOUT=120" >> /etc/profile
	fi
}

function check_enable_status()
{
	LAST_STATUS_VALUE=""
	if [ -f ${DS_STATE_STATUS_FILE} ];then
		LAST_STATUS_VALUE=`head -n 1 ${DS_STATE_STATUS_FILE} 2>/dev/null`
	else
		# 兼容旧版本
		cp ${DS_STATE_ENABLE_FILE} ${DS_STATE_STATUS_FILE} > /dev/null 2>&1
		# 换行，防止新的状态混在第一行
		echo "" | tee -a ${DS_STATE_STATUS_FILE} > /dev/null 2>&1
		LAST_STATUS_VALUE=`head -n 1 ${DS_STATE_STATUS_FILE} 2>/dev/null`
	fi
	LAST_PID="0"
	if [ -f ${DS_STATE_PID_FILE} ];then
		LAST_PID=`cat ${DS_STATE_PID_FILE} 2>/dev/null`
	fi

	# 检查是否有当前进程以外的正在运行的脚本
	IS_LAST_SCRIPT_RUNNING=`ps -ef | grep $SCRIPT_NAME | grep -v " $SCRIPT_PID " | grep -v grep`

	case "$1" in
	enable)
		# 已经开启了等保，直接退出即可
		if [ "$LAST_STATUS_VALUE" = "$STATUS_OPEN" ];then
			echo -n 0
			exit 0
		fi

		# 状态正常，走正常开启等保流程即可
		if [ "$LAST_STATUS_VALUE" = "$STATUS_CLOSE" ];then
			return 0
		fi

		# 状态异常，上次脚本正在开启等保中，需要等待上次脚本关闭等保完成后再开启等保
		if [ "$LAST_STATUS_VALUE" = "$STATUS_CLOSING" ];then
			echo -n '1'
			exit 0
		fi

		# 查看上次运行的脚本的PID文件是否存在
		if [ "$LAST_PID" = "0" ];then
			# 若不存在，则说明上次的脚本必然还没正式开始运行或者已经完成，因此可能有三种情况：
			# 1.上次的脚本刚开始，还未来得及创建PID文件或者结束之前刚删除PID文件，本次脚本进程就被执行了
			# 2.上次的脚本刚开始，还未来得及创建PID文件，就被异常中断了
			# 3.上次的脚本刚执行结束，刚删除了PID文件，就被异常中断了
			if [ -z "$IS_LAST_SCRIPT_RUNNING" ];then
				# 再读一次，尽量保证读的是最新值
				LAST_STATUS_VALUE=`head -n 1 ${DS_STATE_STATUS_FILE} 2>/dev/null`
				if [ "$LAST_STATUS_VALUE" = "$STATUS_OPEN" ];then
					#此为情况3，说明上次运行的脚本已经正常执行完最后一步或已经执行完毕了，正常退出即可
					echo -n 0
					exit 0
				else
					#此为情况2，可看做本次就是第一次尝试启动等保，因此直接开启即可
					return 0
				fi
			else
				# 此处为情况1，不管是PID未创建还是PID刚被删除，说明另一个正在运行的脚本未发生异常，不需要本次进程来开启等保，因此直接返错即可
				echo -n '1'
				exit 0
			fi
		else
			# 若存在，则说明上次运行的脚本已经正式开始了,且必然还没正常运行结束，此时存在两种情况：
			# 1.上次运行的脚本还在运行中，本次脚本进程就被执行了
			# 2.上次运行的脚本已经不在运行中了, 那么本次脚本就需要完成上次脚本未完成的工作，或者全部重来
			IS_PID_RUNNING=`ps -ef | grep " $LAST_PID " | grep $SCRIPT_NAME | grep -v grep`
			if [ -n "$IS_PID_RUNNING" ];then
				# 再读一次，尽量保证读的是最新值
				LAST_STATUS_VALUE=`head -n 1 ${DS_STATE_STATUS_FILE} 2>/dev/null`
				# 此为情况1，可能是上次脚本还在正常运行中，也可能是被异常阻塞了
				if [ "$LAST_STATUS_VALUE" = "$STATUS_OPEN" ];then
					# 此处是唯一能判断上次脚本未被阻塞的情况，此时上次脚本已经正常运行结束了，本次脚本直接退出即可
					echo -n 0
					exit 0
				else
					# 此处因为没法判断上次脚本到底是否被异常阻塞了，所以要杀死上次脚本的进程，重新运行
					pkill -P $LAST_PID > /dev/null 2>&1
					sleep 1
					var_tmp=1
					while true; do
						IS_PID_RUNNING=`ps -ef | grep " $LAST_PID " | grep $SCRIPT_NAME | grep -v grep`
						if [ -z "$IS_PID_RUNNING" ]; then
							break
						fi
						var_tmp=`expr $var + 1 2>/dev/null`
						if [ $var_tmp -gt 5 ]; then
							break
						fi
						sleep 1
					done
					# 杀死上次脚本的进程后，此时情况就变成了情况2，因此直接往后执行情况2的代码即可
				fi
			fi
			# 恢复环境到开启等保前，重新尝试开启等保
			restore_all
			return 0
		fi
		;;
	disable)
		# 已经关闭了等保，直接退出即可
		if [ "$LAST_STATUS_VALUE" = "$STATUS_CLOSE" ];then
			echo -n 0
			exit 0
		fi

		# 状态正常，走正常关闭等保流程即可
		if [ "$LAST_STATUS_VALUE" = "$STATUS_OPEN" ];then
			return 0
		fi

		# 状态异常，上次脚本正在开启等保中，需要等待上次脚本开启等保完成后再关闭等保
		if [ "$LAST_STATUS_VALUE" = "$STATUS_OPENING" ];then
			echo -n '1'
			exit 0
		fi

		# 查看上次运行的脚本的PID文件是否存在
		if [ "$LAST_PID" = "0" ];then
			# 若不存在，则说明上次的脚本必然还没正式开始运行或者已经完成，因此可能有三种情况：
			# 1.上次的脚本刚开始，还未来得及创建PID文件或者结束之前刚删除PID文件，本次脚本进程就被执行了
			# 2.上次的脚本没开始(即本次为close状态下第一次启动等保)，或者刚开始，还未来得及创建PID文件，就被异常中断了
			# 3.上次的脚本刚执行结束，刚删除了PID文件，就被异常中断了
			if [ -z "$IS_LAST_SCRIPT_RUNNING" ];then
				# 再读一次，尽量保证读的是最新值
				LAST_STATUS_VALUE=`head -n 1 ${DS_STATE_STATUS_FILE} 2>/dev/null`
				if [ "$LAST_STATUS_VALUE" = "$STATUS_CLOSE" ];then
					#此为情况3，说明上次运行的脚本已经正常执行完最后一步或已经执行完毕了，正常退出即可
					echo -n 0
					exit 0
				else
					#此为情况2，可看做本次就是第一次尝试启动等保，因此直接开启即可
					return 0
				fi
			else
				# 此处为情况1，不管是PID未创建还是PID刚被删除，说明另一个正在运行的脚本未发生异常，不需要本次进程来开启等保，因此直接返错即可
				echo -n '1'
				exit 0
			fi
		else
			# 若存在，则说明上次运行的脚本已经正式开始了,且必然还没正常运行结束，此时存在两种情况：
			# 1.上次运行的脚本还在运行中，本次脚本进程就被执行了
			# 2.上次运行的脚本已经不在运行中了, 那么本次脚本就需要完成上次脚本未完成的工作，或者全部重来
			IS_PID_RUNNING=`ps -ef | grep " $LAST_PID " | grep $SCRIPT_NAME | grep -v grep`
			if [ -n "$IS_PID_RUNNING" ];then
				# 再读一次，尽量保证读的是最新值
				LAST_STATUS_VALUE=`head -n 1 ${DS_STATE_STATUS_FILE} 2>/dev/null`
				# 此为情况1，可能是上次脚本还在正常运行中，也可能是被异常阻塞了
				if [ "$LAST_STATUS_VALUE" = "$STATUS_CLOSE" ];then
					# 此处是唯一能判断上次脚本未被阻塞的情况，此时上次脚本已经正常运行结束了，本次脚本直接退出即可
					echo -n 0
					exit 0
				else
					# 此处因为没法判断上次脚本到底是否被异常阻塞了，所以要杀死上次脚本的进程，重新运行
					pkill -P $LAST_PID > /dev/null 2>&1
					sleep 1
					var_tmp=1
					while true; do
						IS_PID_RUNNING=`ps -ef | grep " $LAST_PID " | grep $SCRIPT_NAME | grep -v grep`
						if [ -z "$IS_PID_RUNNING" ]; then
							break
						fi
						var_tmp=`expr $var + 1 2>/dev/null`
						if [ $var_tmp -gt 5 ]; then
							break
						fi
						sleep 1
					done
					# 杀死上次脚本的进程后，此时情况就变成了情况2，因此直接往后执行情况2的代码即可
				fi
			fi
			# 重新尝试关闭等保
			return 0
		fi
		;;
	esac
}

case "$1" in
	enable)
		# 将需要修改的的配置文件备份
		back_up_config

		# 检查运行环境
		check_enable_status "enable"

		#保存脚本程序的pid和状态
		echo -n "$SCRIPT_PID" | tee ${DS_STATE_PID_FILE} > /dev/null 2>&1
		echo "$STATUS_OPENING" | tee ${DS_STATE_STATUS_FILE} > /dev/null 2>&1

		################ 身份鉴别 ################
		identity_authentication

		################ 自主访问控制 ################
		# linux本身就满足，不需要加固

		################ 标记和强制访问控制 ################
		#第一阶段，创建管理员账号，并设置密码
		label_and_MAC 1
		#第二阶段，设置管理员账号MAC权限，初步开启三权分立
		label_and_MAC 2
		#第三阶段，设置管理员账号DAC权限，完成三权分立
		label_and_MAC 3
		#第四阶段, 设置打印水印 (仅等保四级要求)
		label_and_MAC 4

		################ 安全审计 ################
		security_audit

		################ 数据完整性 ################
		# 本章节均为功能要求，且依赖硬件，无需也无法通过本加固脚本配置
		#

		################ 数据保密性 ################
		# 本章节均为功能要求，且依赖硬件以及内核配置，无需也无法通过本加固脚本配置
		#

		################ 网络安全保护 ################
		# 本章节均为功能要求，只要提供功能就行，具体配置没有要求，故不需要也无法给出默认配置
		# 需要由系统管理员根据实际环境配置防火墙等
		#

		################ 运行安全保护 ################
		# 启用sysrq,这是等保四级要求
		if [ $SECURITY_LEVEL_FOUR = "true" ];then
			if [ -f /etc/sysctl.conf ];then
				sed -i '/kernel.sysrq=/d' /etc/sysctl.conf
				echo "kernel.sysrq=1" >> /etc/sysctl.conf

				sed -i '/vm.panic_on_oom=/d' /etc/sysctl.conf
				echo "vm.panic_on_oom=1" >> /etc/sysctl.conf

				sed -i '/kernel.panic=/d' /etc/sysctl.conf
				echo "kernel.panic=10" >> /etc/sysctl.conf
			else
				echo -n '-801'
				exit_and_restore
			fi
		fi

		# 限制控制台使用（仅提供机制，需要用户自己根据用户环境的实际情况和需求来配置）

		################ 资源利用 ################
		#本章节均为机制要求，只要提供机制就行，具体配置没有要求（也没法要求，因为这完全是根据用户自己的实际需要来设置的），故不需要也无法给出默认配置
		#比如：/etc/security/limits.conf可以给每个用户限定分配的资源，但是因系统硬件资源以及用户实际需要的资源无法预知，故不可能给出合理的默认配置

		################ 用户登陆访问控制 ################
		login_access_control

		################ 可信度量 ################
		#需要硬件支持，无法配置

		################ 开启SELinux ################
		label_and_MAC 5

		echo "$STATUS_OPEN" | tee ${DS_STATE_STATUS_FILE} > /dev/null 2>&1
		rm ${DS_STATE_PID_FILE} > /dev/null 2>&1
	;;

	disable)
		setenforce 0 > /dev/null 2>&1

		# 检查运行环境
		check_enable_status "disable"

		#保存脚本程序的pid和状态
		echo -n "$SCRIPT_PID" | tee ${DS_STATE_PID_FILE} > /dev/null 2>&1
		echo "$STATUS_CLOSING" | tee ${DS_STATE_STATUS_FILE} > /dev/null 2>&1

		# 修改grub
		rm ${GRUB_CFG_DIR}/${GRUB_CMDLINE_SELINUX_FILE} > /dev/null 2>&1
		update-grub > /dev/null 2>&1
		if [ $? != 0 ];then
			echo "GRUB_CMDLINE_LINUX_DEFAULT=\"\$GRUB_CMDLINE_LINUX_DEFAULT security=selinux checkreqprot=1\"" | tee ${GRUB_CFG_DIR}/${GRUB_CMDLINE_SELINUX_FILE} > /dev/null 2>&1
			if [ $SELINUX_STATUS = "Enforcing" ];then
				setenforce 1 > /dev/null 2>&1
			fi
			echo -n '-323'
			exit 0
		fi

		################ 关闭SELinux ################
		# 开启黑名单模式
		setsebool -P open_deepin_blist 1 > /dev/null 2>&1

		# 此时,SELinux已经关闭了，后续只是还原一些配置，没必要再返错了
		restore_root

		delete_account $2

		# 配置还原必须放最后，保证配置还原后不会再误修改（除了下一步的特例）
		restore_config

		echo "$STATUS_CLOSE" | tee ${DS_STATE_STATUS_FILE} > /dev/null 2>&1
		rm ${DS_STATE_PID_FILE} > /dev/null 2>&1
	;;
esac

echo -n 0